@@ -20,8 +20,14 @@
   (c) 1999, 2000, 2001, 2002 Thomas Lumley.
   2000 Saikat DebRoy
 
-  The format of Stata files is documented under 'file formats'
-  in the Stata manual.
+  Patches by the R Core Team and Brian Quistorff.
+
+  Frozen 2013-10-07 at Stata version 10 aka 114 (11 is the same as 10,
+  and 12 seems compatible despite being described separately as 115).
+
+  The format of Stata files is documented under 'file formats' in the
+  Stata manual and at http://www.stata.com/help.cgi?dta (with,
+  currently, links to various versions).
 
   This code currently does not make use of the print format information in
    a .dta file (except for dates). It cannot handle files with 'int'
@@ -50,7 +56,7 @@
 
 /*
 http://statacorp.com/statalist/archive/2007-06/msg01021.html
-says 113 is versions 8-9, 114 is version 10.
+says 113 is versions 8-9, 114 is version 10/11.
 */
 
 /* Stata format constants */
@@ -188,8 +194,8 @@ SEXP R_LoadStataData(FILE *fp)
 
     /** first read the header **/
 
-    abyte = RawByteBinary(fp, 1);   /* release version */
-    version = 0;			/* -Wall */
+    abyte = (unsigned char) RawByteBinary(fp, 1);   /* release version */
+    version = 0;		/* -Wall */
     varnamelength = 0;		/* -Wall */
     labeltable = R_NilValue;	/* -Wall */
     switch (abyte) {
@@ -274,7 +280,7 @@ SEXP R_LoadStataData(FILE *fp)
     PROTECT(types = allocVector(INTSXP, nvar));
     if (version > 0){
 	for(i = 0; i < nvar; i++){
-	    abyte = RawByteBinary(fp, 1);
+	    abyte = (unsigned char) RawByteBinary(fp, 1);
 	    INTEGER(types)[i] = abyte;
 	    switch (abyte) {
 	    case STATA_FLOAT:
@@ -295,7 +301,7 @@ SEXP R_LoadStataData(FILE *fp)
 	}
     } else {
 	for(i = 0; i < nvar; i++){
-	    abyte = RawByteBinary(fp, 1);
+	    abyte = (unsigned char) RawByteBinary(fp, 1);
 	    INTEGER(types)[i] = abyte;
 	    switch (abyte) {
 	    case STATA_SE_FLOAT:
@@ -497,14 +503,22 @@ SEXP R_LoadStataData(FILE *fp)
 
     /** value labels **/
     if (abs(version) > 5) {
-	/* There may be up to nvar value labels, but possibly 0 */
-	PROTECT(labeltable = allocVector(VECSXP, nvar));
-	PROTECT(tmp = allocVector(STRSXP, nvar));
-	for(j = 0; j < nvar; j++) {
+	PROTECT(labeltable = allocVector(VECSXP, 0));
+	PROTECT(tmp = allocVector(STRSXP, 0));
+	for(j = 0; ; j++) {
 	    /* first int not needed, use fread directly to trigger EOF */
-	    res = fread((int *) aname, sizeof(int), 1, fp);
+	    res = (int) fread((int *) aname, sizeof(int), 1, fp);
 	    if (feof(fp)) break;
 	    if (res != 1) warning(_("a binary read error occurred"));
+
+	    //resize the vectors
+	    labeltable = lengthgets(labeltable, j+1);
+	    UNPROTECT(1);
+	    PROTECT(labeltable);
+	    tmp = lengthgets(tmp, j+1);
+	    UNPROTECT(1);
+	    PROTECT(tmp);
+
 	    InStringBinary(fp, varnamelength+1, aname);
 	    SET_STRING_ELT(tmp, j, mkChar(aname));
 	    RawByteBinary(fp, 1); RawByteBinary(fp, 1); RawByteBinary(fp, 1); /*padding*/
@@ -516,7 +530,7 @@ SEXP R_LoadStataData(FILE *fp)
 	    for(i = 0; i < nlabels; i++)
 		off[i] = InIntegerBinary(fp, 1, swapends);
 	    for(i = 0; i < nlabels; i++)
-		INTEGER(levels)[i] = (double) InIntegerBinary(fp, 0, swapends);
+		INTEGER(levels)[i] = InIntegerBinary(fp, 0, swapends);
 	    txt =  Calloc((size_t) totlen, char);
 	    InStringBinary(fp, totlen, txt);
 	    for(i = 0; i < nlabels; i++)
@@ -529,11 +543,6 @@ SEXP R_LoadStataData(FILE *fp)
 	}
 	namesgets(labeltable, tmp);
 	UNPROTECT(1); /*tmp*/
-	if(j > 0 && j < nvar) {
-	    labeltable = lengthgets(labeltable, j);
-	    UNPROTECT(1);
-	    PROTECT(labeltable);
-	}
     }
 
     /** tidy up **/
@@ -611,11 +620,11 @@ static void OutShortIntBinary(int i,FILE * fp)
   unsigned char first,second;
 
 #ifdef WORDS_BIGENDIAN
-    first= (i>>8);
-    second=i & 0xff;
+    first = (i >> 8);
+    second = i & 0xff;
 #else
-    first=i & 0xff;
-    second=i>>8;
+    first = i & 0xff;
+    second = i >> 8;
 #endif
   if (fwrite(&first, sizeof(char), 1, fp) != 1)
     error(_("a binary write error occurred"));
@@ -655,7 +664,8 @@ static Rboolean
 writeStataValueLabel(const char *labelName, const SEXP theselabels,
 		     const SEXP theselevels, const int namelength, FILE *fp)
 {
-    int i,len,txtlen; 
+    int i, txtlen; 
+    size_t len;
 
     if(!isString(theselabels))
 	return FALSE;
@@ -719,7 +729,7 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
     char datalabel[81]="Written by R.              ", timestamp[18], aname[33];
     char format9g[50]="%9.0g", strformat[50]="";
     const char *thisnamechar;
-    SEXP names,types,theselabels,orig_names,dlabel,exp_fields,exp_field,curr_val_labels,label_table,names_lt,theselabelslevels;
+    SEXP names,types,theselabels,orig_names,vlabels,dlabel,exp_fields,exp_field,curr_val_labels,label_table,names_lt,theselabelslevels;
 
     int namelength = 8;
     int fmtlist_len = 12;
@@ -785,7 +795,7 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
 		/* NB: there is a 244 byte limit on strings */
 		charlen = 0;
 		for(j = 0; j < nobs; j++){
-		    k = strlen(CHAR(STRING_ELT(VECTOR_ELT(df, i), j)));
+		    k = (int) strlen(CHAR(STRING_ELT(VECTOR_ELT(df, i), j)));
 		    if (k > charlen) charlen = k;
 		}
 		if(charlen > 244)
@@ -815,7 +825,7 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
 		/* NB: there is a 244 byte limit on strings */
 		charlen = 0;
 		for(j = 0;j < nobs; j++){
-		    k = strlen(CHAR(STRING_ELT(VECTOR_ELT(df, i),j)));
+		    k = (int) strlen(CHAR(STRING_ELT(VECTOR_ELT(df, i),j)));
 		    if (k > charlen) charlen = k;
 		}
 		if(charlen > 244)
@@ -879,14 +889,25 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
     UNPROTECT(1);
 
 
-    /** Variable Labels -- full R name of column**/
-    /** FIXME: this is now just the same abbreviated name **/
-
-    PROTECT(orig_names = getAttrib(df,install("orig.names")));
-    for(i = 0; i < nvar; i++) {
-	strncpy(datalabel,CHAR(STRING_ELT(orig_names,i)),81);
-	datalabel[80] = (char) 0;
-	OutStringBinary(datalabel, fp, 81);
+    /** Variable Labels -- Uses "var.labels" attribute
+	if is a string vector of the right length, otherwise the
+	the variable name (FIXME: this is now just the same abbreviated name) **/
+    PROTECT(vlabels = getAttrib(df, install("var.labels")));
+    if(!isNull(vlabels) && isString(vlabels) && LENGTH(vlabels)==nvar){
+        for(i = 0; i < nvar; i++) {
+	    strncpy(datalabel,CHAR(STRING_ELT(vlabels,i)),81);
+	    datalabel[80] = (char) 0;
+	    OutStringBinary(datalabel, fp, 81);
+        }
+    }
+    else{
+        PROTECT(orig_names = getAttrib(df,install("orig.names")));
+        for(i = 0; i < nvar; i++) {
+	    strncpy(datalabel,CHAR(STRING_ELT(orig_names,i)),81);
+	    datalabel[80] = (char) 0;
+	    OutStringBinary(datalabel, fp, 81);
+        }
+        UNPROTECT(1);
     }
     UNPROTECT(1);
 
@@ -940,6 +961,8 @@ void R_SaveStataData(FILE *fp, SEXP df, int version, SEXP leveltable)
 	    case STRSXP:
 		/* Up to 244 bytes should be written, zero-padded */
 		k = length(STRING_ELT(VECTOR_ELT(df, j), i));
+		if (k == 0)
+		    error("empty string is not valid in Stata's documented format");
 		if(k > 244) k = 244;
 		OutStringBinary(CHAR(STRING_ELT(VECTOR_ELT(df, j), i)), fp, k);
 		for(l = INTEGER(types)[j]-k; l > 0; l--) OutByteBinary(0, fp);
